@@ -379,6 +379,7 @@ static void testDownloadLocalFileError(DownloadErrorTest* test, gconstpointer)
 
 static WebKitTestServer* kServer;
 static const char* kServerSuggestedFilename = "webkit-downloaded-file";
+static HashMap<CString, CString> s_userAgentMap;
 
 static void addContentDispositionHTTPHeaderToResponse(SoupMessage* message)
 {
@@ -408,6 +409,9 @@ static void serverCallback(SoupServer* server, SoupMessage* message, const char*
 
     soup_message_set_status(message, SOUP_STATUS_OK);
 
+    if (g_str_has_prefix(path, "/ua-"))
+        s_userAgentMap.add(path, soup_message_headers_get_one(message->request_headers, "User-Agent"));
+
     if (g_str_equal(path, "/cancel-after-destination")) {
         // Use an infinite message to make sure it's cancelled before it finishes.
         soup_message_headers_set_encoding(message->response_headers, SOUP_ENCODING_CHUNKED);
@@ -417,7 +421,13 @@ static void serverCallback(SoupServer* server, SoupMessage* message, const char*
         return;
     }
 
-    if (g_str_equal(path, "/unknown"))
+    if (g_str_equal(path, "/ua-test-redirect")) {
+        soup_message_set_status(message, SOUP_STATUS_MOVED_PERMANENTLY);
+        soup_message_headers_append(message->response_headers, "Location", "/ua-test");
+        return;
+    }
+
+    if (g_str_equal(path, "/unknown") || g_str_equal(path, "/ua-test"))
         path = "/test.pdf";
 
     GUniquePtr<char> filePath(g_build_filename(Test::getResourcesDir().data(), path, nullptr));
@@ -559,8 +569,41 @@ public:
         g_main_loop_run(m_mainLoop);
     }
 
+#if PLATFORM(GTK)
+    static gboolean contextMenuCallback(WebKitWebView* webView, WebKitContextMenu* contextMenu, GdkEvent*, WebKitHitTestResult* hitTestResult, WebViewDownloadTest* test)
+    {
+        g_assert_true(WEBKIT_IS_HIT_TEST_RESULT(hitTestResult));
+        GList* items = webkit_context_menu_get_items(contextMenu);
+        for (GList* l = items; l; l = g_list_next(l)) {
+            g_assert_true(WEBKIT_IS_CONTEXT_MENU_ITEM(l->data));
+            auto* item = WEBKIT_CONTEXT_MENU_ITEM(l->data);
+            if (webkit_context_menu_item_get_stock_action(item) == WEBKIT_CONTEXT_MENU_ACTION_DOWNLOAD_LINK_TO_DISK) {
+                test->m_contextMenuDownloadItem = item;
+                break;
+            }
+        }
+        test->quitMainLoop();
+        return FALSE;
+    }
+
+    WebKitContextMenuItem* showContextMenuAndGetDownloadItem(int x, int y)
+    {
+        m_contextMenuDownloadItem = nullptr;
+        auto id = g_signal_connect(m_webView, "context-menu", G_CALLBACK(contextMenuCallback), this);
+        RunLoop::main().dispatch([this, x, y] {
+            clickMouseButton(x, y, 3);
+        });
+        g_main_loop_run(m_mainLoop);
+        g_signal_handler_disconnect(m_webView, id);
+        return m_contextMenuDownloadItem.get();
+    }
+#endif
+
     GRefPtr<WebKitDownload> m_download;
     bool m_shouldDelayDecideDestination { false };
+#if PLATFORM(GTK)
+    GRefPtr<WebKitContextMenuItem> m_contextMenuDownloadItem;
+#endif
 };
 
 static void testWebViewDownloadURI(WebViewDownloadTest* test, gconstpointer)
@@ -688,52 +731,56 @@ static void testDownloadMIMEType(DownloadTest* test, gconstpointer)
     test->checkDestinationAndDeleteFile(download.get(), expectedFilename.get());
 }
 
-#if PLATFORM(GTK)
-static gboolean contextMenuCallback(WebKitWebView* webView, WebKitContextMenu* contextMenu, GdkEvent*, WebKitHitTestResult* hitTestResult, WebViewDownloadTest* test)
+static void testDownloadUserAgent(DownloadTest* test, gconstpointer)
 {
-    g_assert_true(WEBKIT_IS_HIT_TEST_RESULT(hitTestResult));
-    g_assert_true(webkit_hit_test_result_context_is_link(hitTestResult));
-    GList* items = webkit_context_menu_get_items(contextMenu);
-    GRefPtr<WebKitContextMenuItem> contextMenuItem;
-    for (GList* l = items; l; l = g_list_next(l)) {
-        g_assert_true(WEBKIT_IS_CONTEXT_MENU_ITEM(l->data));
-        auto* item = WEBKIT_CONTEXT_MENU_ITEM(l->data);
-        if (webkit_context_menu_item_get_stock_action(item) == WEBKIT_CONTEXT_MENU_ACTION_DOWNLOAD_LINK_TO_DISK) {
-            contextMenuItem = item;
-            break;
-        }
-    }
-    g_assert_nonnull(contextMenuItem.get());
-    webkit_context_menu_remove_all(contextMenu);
-    webkit_context_menu_append(contextMenu, contextMenuItem.get());
-    test->quitMainLoop();
-    return FALSE;
+    s_userAgentMap.clear();
+    GRefPtr<WebKitDownload> download = test->downloadURIAndWaitUntilFinishes(kServer->getURIForPath("/ua-test"));
+    g_assert_null(webkit_download_get_web_view(download.get()));
+    WebKitURIRequest* request = webkit_download_get_request(download.get());
+    g_assert_true(WEBKIT_IS_URI_REQUEST(request));
+    ASSERT_CMP_CSTRING(webkit_uri_request_get_uri(request), ==, kServer->getURIForPath("/ua-test"));
+
+    const char* userAgent = soup_message_headers_get_one(webkit_uri_request_get_http_headers(request), "User-Agent");
+    g_assert_nonnull(userAgent);
+    g_assert_cmpuint(s_userAgentMap.size(), ==, 1);
+    g_assert_true(s_userAgentMap.contains("/ua-test"));
+    ASSERT_CMP_CSTRING(userAgent, ==, s_userAgentMap.get("/ua-test"));
+    s_userAgentMap.clear();
+
+    GUniquePtr<char> expectedFilename(g_strdup_printf("%s.pdf", kServerSuggestedFilename));
+    test->checkDestinationAndDeleteFile(download.get(), expectedFilename.get());
+
+    download = test->downloadURIAndWaitUntilFinishes(kServer->getURIForPath("/ua-test-redirect"));
+    g_assert_null(webkit_download_get_web_view(download.get()));
+    request = webkit_download_get_request(download.get());
+    g_assert_true(WEBKIT_IS_URI_REQUEST(request));
+    ASSERT_CMP_CSTRING(webkit_uri_request_get_uri(request), ==, kServer->getURIForPath("/ua-test-redirect"));
+
+    userAgent = soup_message_headers_get_one(webkit_uri_request_get_http_headers(request), "User-Agent");
+    g_assert_nonnull(userAgent);
+    g_assert_cmpuint(s_userAgentMap.size(), ==, 2);
+    g_assert_true(s_userAgentMap.contains("/ua-test-redirect"));
+    ASSERT_CMP_CSTRING(userAgent, ==, s_userAgentMap.get("/ua-test-redirect"));
+    g_assert_true(s_userAgentMap.contains("/ua-test"));
+    ASSERT_CMP_CSTRING(userAgent, ==, s_userAgentMap.get("/ua-test"));
+    s_userAgentMap.clear();
+
+    test->checkDestinationAndDeleteFile(download.get(), expectedFilename.get());
 }
 
+#if PLATFORM(GTK)
 static void testContextMenuDownloadActions(WebViewDownloadTest* test, gconstpointer)
 {
-    test->showInWindowAndWaitUntilMapped();
+    test->showInWindow();
 
     static const char* linkHTMLFormat = "<html><body><a style='position:absolute; left:1; top:1' href='%s'>Download Me</a></body></html>";
     GUniquePtr<char> linkHTML(g_strdup_printf(linkHTMLFormat, kServer->getURIForPath("/test.pdf").data()));
     test->loadHtml(linkHTML.get(), kServer->getURIForPath("/").data());
     test->waitUntilLoadFinished();
 
-    g_signal_connect(test->m_webView, "context-menu", G_CALLBACK(contextMenuCallback), test);
-    g_idle_add([](gpointer userData) -> gboolean {
-        auto* test = static_cast<WebViewDownloadTest*>(userData);
-        test->clickMouseButton(1, 1, 3);
-        return FALSE;
-    }, test);
-    g_main_loop_run(test->m_mainLoop);
-
-    g_idle_add([](gpointer userData) -> gboolean {
-        auto* test = static_cast<WebViewDownloadTest*>(userData);
-        // Select and activate the context menu action.
-        test->keyStroke(GDK_KEY_Down);
-        test->keyStroke(GDK_KEY_Return);
-        return FALSE;
-    }, test);
+    auto* item = test->showContextMenuAndGetDownloadItem(1, 1);
+    g_assert(WEBKIT_IS_CONTEXT_MENU_ITEM(item));
+    g_action_activate(webkit_context_menu_item_get_gaction(item), nullptr);
     test->waitUntilDownloadStarted();
 
     g_assert_true(test->m_webView == webkit_download_get_web_view(test->m_download.get()));
@@ -755,7 +802,7 @@ static void testContextMenuDownloadActions(WebViewDownloadTest* test, gconstpoin
 
 static void testBlobDownload(WebViewDownloadTest* test, gconstpointer)
 {
-    test->showInWindowAndWaitUntilMapped();
+    test->showInWindow();
 
     static const char* linkBlobHTML =
         "<html><body>"
@@ -805,9 +852,13 @@ void beforeAll()
     PolicyResponseDownloadTest::add("Downloads", "policy-decision-download", testPolicyResponseDownload);
     PolicyResponseDownloadTest::add("Downloads", "policy-decision-download-cancel", testPolicyResponseDownloadCancel);
     DownloadTest::add("Downloads", "mime-type", testDownloadMIMEType);
+    DownloadTest::add("Downloads", "user-agent", testDownloadUserAgent);
     // FIXME: Implement keyStroke in WPE.
 #if PLATFORM(GTK)
+#if !USE(GTK4)
+    // FIXME: Rework context menu API in GTK4 to not expose GdkEvent.
     WebViewDownloadTest::add("Downloads", "contex-menu-download-actions", testContextMenuDownloadActions);
+#endif
     // FIXME: Implement mouse click in WPE.
     WebViewDownloadTest::add("Downloads", "blob-download", testBlobDownload);
 #endif
